import logging
from ortu.oci_auth import get_network_client
from oci.core.models import UpdateRouteTableDetails, RouteRule


_log = logging.getLogger(name=__name__)


def _construct_route_rule(cidr, ne_ocid):
    route_rule = RouteRule(
        destination=cidr,
        destination_type="CIDR_BLOCK",
        network_entity_id=ne_ocid
    )
    return route_rule


def get_route_table(rt_ocid):
    network_client = get_network_client()
    return network_client.get_route_table(rt_ocid).data


def check_route(route_table, cidr, ne_ocid):
    route_rule = _construct_route_rule(cidr, ne_ocid)
    _log.debug(f"Route rule: {route_rule}")

    filtered_rules = list(filter(lambda rule: rule.destination == cidr and rule.network_entity_id == ne_ocid,
                                 route_table.route_rules))

    if filtered_rules:
        return True
    else:
        return False


def update_route_table(route_table, cidr, ne_ocid, dry_run):
    network_client = get_network_client()

    route_rules = route_table.route_rules.copy()
    route_rules.append(_construct_route_rule(cidr, ne_ocid))

    update_route_table_details = UpdateRouteTableDetails(
        route_rules=route_rules
    )

    if not dry_run:
        network_client.update_route_table(route_table.id, update_route_table_details=update_route_table_details)
    else:
        _log.debug(f"Route table update details: {update_route_table_details}")



from os import environ, path
from oci import config, core


def _get_oci_config():
    oci_config_file = path.join(path.expanduser("~"), ".oci", "config")
    oci_config_profile = 'DEFAULT'

    if "OCI_CONFIG_PROFILE" in environ:
        oci_config_profile = environ.get("OCI_CONFIG_PROFILE")

    return config.from_file(file_location=oci_config_file, profile_name=oci_config_profile)


def get_network_client():
    oci_config = _get_oci_config()
    return core.VirtualNetworkClient(oci_config)



from ortu.oci_route_table import check_route
from oci.core.models import RouteTable, RouteRule


SOURCE_ROUTE_RULES = [
    RouteRule(
        destination='0.0.0.0/0',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.internetgateway.oc1.region.somelongstring'
    )
]

SOURCE_ROUTE_TABLE = RouteTable(
    route_rules=SOURCE_ROUTE_RULES
)

NEW_ROUTE_RULES = [
    RouteRule(
        destination='0.0.0.0/0',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.internetgateway.oc1.region.somelongstring'
    ),
    RouteRule(
        destination='192.168.1.0/24',
        destination_type='CIDR_BLOCK',
        network_entity_id='ocid1.internetgateway.oc1.region.anotherlongcrazystring'
    )
]

NEW_ROUTE_TABLE = RouteTable(
    route_rules=NEW_ROUTE_RULES
)


class Args:
    def __init__(self):
        pass

    cidr = '192.168.1.0/24'
    ne_ocid = 'ocid1.internetgateway.oc1.region.anotherlongcrazystring'


args = Args


def test_check_update_needed():
    assert not check_route(SOURCE_ROUTE_TABLE, args.cidr, args.ne_ocid)


def test_check_update_not_needed():
    assert check_route(NEW_ROUTE_TABLE, args.cidr, args.ne_ocid)
